@@ -2,6 +2,7 @@ import argparse
 import itertools
 import json
 import logging
+import matplotlib.pyplot as plt
 import numpy as np
 import os
 import warnings
@@ -18,25 +19,32 @@ from sacrerouge.io import JsonlReader
 logger = logging.getLogger(__name__)
 
 
-def load_metrics(metrics_files: List[str]) -> List[Metrics]:
+def _load_metrics(metrics_files: List[str]) -> List[Metrics]:
     logger.info(f'Loading metrics from {metrics_files}')
-    metrics_dicts = defaultdict(dict)
-    for metrics_file in metrics_files:
-        with JsonlReader(metrics_file, Metrics) as f:
-            for metrics in f:
-                if metrics.summarizer_id not in metrics_dicts[metrics.instance_id]:
-                    metrics_dicts[metrics.instance_id][metrics.summarizer_id] = metrics
-                else:
-                    metrics_dicts[metrics.instance_id][metrics.summarizer_id].merge(metrics)
-
     metrics_list = []
-    for metrics_dict in metrics_dicts.values():
-        metrics_list.extend(list(metrics_dict.values()))
-    logger.info(f'Loaded {len(metrics_list)} instances')
+    for metrics_file in metrics_files:
+        metrics_list.extend(JsonlReader(metrics_file, Metrics).read())
+    logger.info(f'Loaded {len(metrics_list)} metrics objects')
     return metrics_list
 
 
-def filter_metrics(metrics_list: List[Metrics], summarizer_type: str, metric1: str, metric2: str):
+def _merge_metrics(metrics_list: List[Metrics]) -> List[Metrics]:
+    logger.info(f'Merging multiple metrics objects for the same (instance, summary) into a single object')
+    metrics_dicts = defaultdict(dict)
+    for metrics in metrics_list:
+        if metrics.summarizer_id not in metrics_dicts[metrics.instance_id]:
+            metrics_dicts[metrics.instance_id][metrics.summarizer_id] = metrics
+        else:
+            metrics_dicts[metrics.instance_id][metrics.summarizer_id].merge(metrics)
+
+    merged_metrics_list = []
+    for metrics_dict in metrics_dicts.values():
+        merged_metrics_list.extend(list(metrics_dict.values()))
+    logger.info(f'Merged into {len(merged_metrics_list)} instances')
+    return merged_metrics_list
+
+
+def _filter_metrics(metrics_list: List[Metrics], summarizer_type: str, metric1: str, metric2: str) -> List[Metrics]:
     logger.info(f'Filtering instances to summarizer type "{summarizer_type}" and metrics "{metric1}" and "{metric2}"')
 
     filtered = []
@@ -75,7 +83,7 @@ def aggregate_metrics(metrics_list: List[Metrics]) -> Dict[str, MetricsDict]:
     return key_to_metrics
 
 
-def compute_summary_level_correlations(metrics_list: List[Dict[str, Any]],
+def compute_summary_level_correlations(metrics_list: List[Metrics],
                                        metric1: str,
                                        metric2: str) -> Tuple[Dict[str, float], Dict[str, Dict[str, float]]]:
     pearsons = {}
@@ -127,7 +135,7 @@ def compute_summary_level_correlations(metrics_list: List[Dict[str, Any]],
     return averaged_correlations, individual_correlations
 
 
-def compute_system_level_correlations(metrics_list: List[Dict[str, Any]],
+def compute_system_level_correlations(metrics_list: List[Metrics],
                                       metric1: str,
                                       metric2: str) -> Dict[str, float]:
     metrics_list = list(aggregate_metrics(metrics_list).values())
@@ -157,7 +165,7 @@ def compute_system_level_correlations(metrics_list: List[Dict[str, Any]],
     }
 
 
-def compute_global_correlations(metrics_list: List[Dict[str, Any]],
+def compute_global_correlations(metrics_list: List[Metrics],
                                 metric1: str,
                                 metric2: str) -> Dict[str, float]:
     values1 = [metrics.metrics[metric1] for metrics in metrics_list]
@@ -185,31 +193,99 @@ def compute_global_correlations(metrics_list: List[Dict[str, Any]],
     }
 
 
-def compute_correlation(metrics_jsonl_files: Union[str, List[str]],
+def _plot_values(values1: List[float],
+                 values2: List[float],
+                 metric1: str,
+                 metric2: str,
+                 label: str,
+                 output_file: str) -> None:
+    fig = plt.figure()
+    plt.xlabel(metric1)
+    plt.ylabel(metric2)
+    plt.scatter(values1, values2, label=label)
+    plt.legend()
+    plt.tight_layout()
+
+    dirname = os.path.dirname(output_file)
+    if dirname:
+        os.makedirs(dirname, exist_ok=True)
+    fig.savefig(output_file)
+    plt.close()
+
+
+def _plot_system_level_metrics(metrics_list: List[Metrics],
+                               metric1: str,
+                               metric2: str,
+                               output_file: str) -> None:
+    metrics_list = list(aggregate_metrics(metrics_list).values())
+    values1 = [metrics[metric1] for metrics in metrics_list]
+    values2 = [metrics[metric2] for metrics in metrics_list]
+    _plot_values(values1, values2, metric1, metric2, 'Systems', output_file)
+
+
+def _plot_global_metrics(metrics_list: List[Metrics],
+                         metric1: str,
+                         metric2: str,
+                         output_file: str) -> None:
+    values1 = [metrics.metrics[metric1] for metrics in metrics_list]
+    values2 = [metrics.metrics[metric2] for metrics in metrics_list]
+    _plot_values(values1, values2, metric1, metric2, 'Summaries', output_file)
+
+
+def compute_correlation(metrics_jsonl_files_or_metrics_list: Union[str, List[str], List[Metrics]],
                         metric1: str,
                         metric2: str,
                         summarizer_type: str,
-                        return_all_summary_level: bool = False):
-    if isinstance(metrics_jsonl_files, str):
-        metrics_jsonl_files = [metrics_jsonl_files]
+                        return_all_summary_level: bool = False,
+                        skip_summary_level: bool = False,
+                        skip_system_level: bool = False,
+                        skip_global: bool = False,
+                        system_level_output_plot: str = None,
+                        global_output_plot: str = None):
+    if return_all_summary_level:
+        assert not skip_summary_level, 'If `return_all_summary_level` is `True`, `skip_summary_level` must not be `True`'
+    if system_level_output_plot is not None:
+        assert not skip_system_level, 'If `system_level_output_plot` is not `None`, system-level correlations must be calculated'
+    if global_output_plot is not None:
+        assert not skip_global, 'If `global_output_plot` is not `None`, global correlations must be calculated'
+
+    if isinstance(metrics_jsonl_files_or_metrics_list, str):
+        # A single file
+        metrics_list = _load_metrics([metrics_jsonl_files_or_metrics_list])
+    elif isinstance(metrics_jsonl_files_or_metrics_list, list) and all(isinstance(item, str) for item in metrics_jsonl_files_or_metrics_list):
+        # A list of files
+        metrics_list = _load_metrics(metrics_jsonl_files_or_metrics_list)
+    else:
+        # A list of metrics
+        assert isinstance(metrics_jsonl_files_or_metrics_list, list) and all(isinstance(item, Metrics) for item in metrics_jsonl_files_or_metrics_list)
+        metrics_list = metrics_jsonl_files_or_metrics_list
+
+    # Merge duplicate metrics objects into one
+    metrics_list = _merge_metrics(metrics_list)
 
-    metrics_list = load_metrics(metrics_jsonl_files)
     for metrics in metrics_list:
         metrics.flatten_keys()
 
-    metrics_list = filter_metrics(metrics_list, summarizer_type, metric1, metric2)
+    metrics_list = _filter_metrics(metrics_list, summarizer_type, metric1, metric2)
     for metrics in metrics_list:
         metrics.select_metrics([metric1, metric2])
         metrics.average_values()
 
-    summary_level, individual_summary_level = compute_summary_level_correlations(metrics_list, metric1, metric2)
-    system_level = compute_system_level_correlations(metrics_list, metric1, metric2)
-    global_level = compute_global_correlations(metrics_list, metric1, metric2)
-    results = {
-        'summary_level': summary_level,
-        'system_level': system_level,
-        'global': global_level
-    }
+    results = {}
+    if not skip_summary_level:
+        summary_level, individual_summary_level = compute_summary_level_correlations(metrics_list, metric1, metric2)
+        results['summary_level'] = summary_level
+
+    if not skip_system_level:
+        results['system_level'] = compute_system_level_correlations(metrics_list, metric1, metric2)
+        if system_level_output_plot is not None:
+            _plot_system_level_metrics(metrics_list, metric1, metric2, system_level_output_plot)
+
+    if not skip_global:
+        results['global'] = compute_global_correlations(metrics_list, metric1, metric2)
+        if global_output_plot is not None:
+            _plot_global_metrics(metrics_list, metric1, metric2, global_output_plot)
+
     if return_all_summary_level:
         results = (results, individual_summary_level)
     return results
@@ -261,6 +337,31 @@ class CorrelateSubcommand(RootSubcommand):
             type=str,
             help='The file where all of the summary-level correlations should be written'
         )
+        self.parser.add_argument(
+            '--skip-summary-level',
+            action='store_true',
+            help='Indicates the summary-level correlations should not be calculated'
+        )
+        self.parser.add_argument(
+            '--skip-system-level',
+            action='store_true',
+            help='Indicates the system-level correlations should not be calculated'
+        )
+        self.parser.add_argument(
+            '--skip-global',
+            action='store_true',
+            help='Indicates the global correlations should not be calculated'
+        )
+        self.parser.add_argument(
+            '--system-level-output-plot',
+            type=str,
+            help='If provided, saves a plot of the system-level scores to the corresponding file'
+        )
+        self.parser.add_argument(
+            '--global-output-plot',
+            type=str,
+            help='If provided, saves a plot of the global scores to the corresponding file'
+        )
         self.parser.set_defaults(func=self.run)
 
     def run(self, args):
@@ -269,7 +370,12 @@ class CorrelateSubcommand(RootSubcommand):
         metric1, metric2 = args.metrics
         return_all_summary_level = args.summary_level_correlations_output is not None
         results = compute_correlation(args.metrics_jsonl_files, metric1, metric2, args.summarizer_type,
-                                      return_all_summary_level=return_all_summary_level)
+                                      return_all_summary_level=return_all_summary_level,
+                                      skip_summary_level=args.skip_summary_level,
+                                      skip_system_level=args.skip_system_level,
+                                      skip_global=args.skip_global,
+                                      system_level_output_plot=args.system_level_output_plot,
+                                      global_output_plot=args.global_output_plot)
 
         # Strip off the original results from the individual summary correlations
         if return_all_summary_level:
