@@ -1,3 +1,4 @@
+import logging
 from typing import List
 
 from sacrerouge.data import EvalInstance
@@ -5,31 +6,32 @@ from sacrerouge.data.dataset_readers import DatasetReader
 from sacrerouge.data.fields import ReferencesField, Fields, SummaryField
 from sacrerouge.io import JsonlReader
 
+logger = logging.getLogger(__name__)
+
 
 @DatasetReader.register('reference-based')
 class ReferenceBasedDatasetReader(DatasetReader):
-    def __init__(self, input_jsonl: str) -> None:
-        super().__init__()
-        self.input_jsonl = input_jsonl
-
-    def read(self) -> List[EvalInstance]:
+    def read(self, input_jsonl: str) -> List[EvalInstance]:
+        logger.info(f'Loading evaluation instances from {input_jsonl}')
         instances = []
-        with JsonlReader(self.input_jsonl) as f:
+        with JsonlReader(input_jsonl) as f:
             for data in f:
-                summary = SummaryField(data['summary']['text'])
+                fields = {}
+                fields['summary'] = SummaryField(data['summary']['text'])
 
                 if 'reference' in data:
-                    references = ReferencesField([data['reference']['text']])
+                    fields['references'] = ReferencesField([data['reference']['text']])
                 else:
-                    references = ReferencesField([reference['text'] for reference in data['references']])
-                fields = Fields({'references': references})
+                    fields['references'] = ReferencesField([reference['text'] for reference in data['references']])
+                fields = Fields(fields)
 
                 instance = EvalInstance(
                     data['instance_id'],
                     data['summarizer_id'],
                     data['summarizer_type'],
-                    summary,
                     fields
                 )
                 instances.append(instance)
+
+            logger.info(f'Loaded {len(instances)} instances')
             return instances
