@@ -1,4 +1,5 @@
 import json
+import os
 import subprocess
 import pytest
 import unittest
@@ -151,3 +152,64 @@ class TestCorrelate(unittest.TestCase):
             assert summary_level['kendall']['M000'] == pytest.approx(0.3333333333333334, abs=1e-4)
             assert summary_level['kendall']['M001'] == pytest.approx(0.18257418583505539, abs=1e-4)
             assert summary_level['kendall']['M002'] == pytest.approx(0.0, abs=1e-4)
+
+    def test_skip_calculations(self):
+        # Ensures the flags to skip calculating specific correlations work
+        with TemporaryDirectory() as temp_dir:
+            command = [
+                'python', '-m', 'sacrerouge', 'correlate',
+                '--metrics-jsonl-files', MULTILING_METRICS,
+                '--metrics', 'rouge-1_jk_precision', 'grade',
+                '--summarizer-type', 'all',
+                '--output-file', f'{temp_dir}/correlations.json',
+                '--silent',
+                '--skip-summary-level'
+            ]
+            subprocess.run(command, check=True)
+            correlations = json.load(open(f'{temp_dir}/correlations.json', 'r'))
+            assert 'summary_level' not in correlations
+
+            command = [
+                'python', '-m', 'sacrerouge', 'correlate',
+                '--metrics-jsonl-files', MULTILING_METRICS,
+                '--metrics', 'rouge-1_jk_precision', 'grade',
+                '--summarizer-type', 'all',
+                '--output-file', f'{temp_dir}/correlations.json',
+                '--silent',
+                '--skip-system-level'
+            ]
+            subprocess.run(command, check=True)
+            correlations = json.load(open(f'{temp_dir}/correlations.json', 'r'))
+            assert 'system_level' not in correlations
+
+            command = [
+                'python', '-m', 'sacrerouge', 'correlate',
+                '--metrics-jsonl-files', MULTILING_METRICS,
+                '--metrics', 'rouge-1_jk_precision', 'grade',
+                '--summarizer-type', 'all',
+                '--output-file', f'{temp_dir}/correlations.json',
+                '--silent',
+                '--skip-global'
+            ]
+            subprocess.run(command, check=True)
+            correlations = json.load(open(f'{temp_dir}/correlations.json', 'r'))
+            assert 'global' not in correlations
+
+    def test_plots(self):
+        # Tests to ensure plot files exist
+        with TemporaryDirectory() as temp_dir:
+            system_plot_file = f'{temp_dir}/system.pdf'
+            global_plot_file = f'{temp_dir}/global.pdf'
+            command = [
+                'python', '-m', 'sacrerouge', 'correlate',
+                '--metrics-jsonl-files', MULTILING_METRICS,
+                '--metrics', 'rouge-1_jk_precision', 'grade',
+                '--summarizer-type', 'all',
+                '--output-file', f'{temp_dir}/correlations.json',
+                '--silent',
+                '--system-level-output-plot', system_plot_file,
+                '--global-output-plot', global_plot_file
+            ]
+            subprocess.run(command, check=True)
+            assert os.path.exists(system_plot_file)
+            assert os.path.exists(global_plot_file)
